==> taboola_sdk-0.2.1.tar.gz: lib/classic/taboola_classic.dart <==
import 'dart:io' show Platform;
import 'package:flutter/material.dart';
import 'package:taboola_sdk/descriptors/device_descriptor.dart';
import 'package:taboola_sdk/classic/webview_manager.dart';
import 'package:taboola_sdk/classic/taboola_webview.dart';
import 'package:taboola_sdk/classic/taboola_classic_listener.dart';
import 'package:webview_flutter/webview_flutter.dart';
import '../extra_properties_manager.dart';
import 'itemBrowser.dart';
import 'package:taboola_sdk/utils/logger.dart';

class TaboolaClassicUnit extends StatefulWidget {
  TaboolaClassicUnit._(
    this._extraPropertiesManager,
    this._publisherId,
    this._mode,
    this._placement,
    this._pageUrl,
    this._pageType,
    this._pageId,
    this._targetType,
    this._isFeed,
    this._taboolaStandardListener, {
    Key key,
    viewId,
    ScrollController scrollController,
  })  : this.scrollController = scrollController ??= ScrollController(),
        this._viewId = viewId ?? new DateTime.now().millisecondsSinceEpoch,
        super(
            key:
                key); //Note: '{} means scrollController is an optional argument, it allows publishers to set an external ScrollController from their scrolling parent. Otherwise, on default, we create an internal listener and ask the user to set it in their scrolling view.'

  factory TaboolaClassicUnit(
    ExtraPropertiesManager extraPropertiesManager,
    String publisherId,
    String mode,
    String placement,
    String pageUrl,
    String pageType,
    String pageId,
    String targetType,
    bool isFeed,
    TaboolaClassicListener taboolaClassicListener, {
    int viewId,
    ScrollController scrollController,
  }) {
    return TaboolaClassicUnit._(
      extraPropertiesManager,
      publisherId,
      mode,
      placement,
      pageUrl,
      pageType,
      pageId,
      targetType,
      isFeed,
      taboolaClassicListener,
      viewId: viewId,
      scrollController: scrollController,
    );
  }

  final ExtraPropertiesManager _extraPropertiesManager;
  final String _publisherId;
  final String _mode;
  final String _placement;
  final String _pageUrl;
  final String _pageType;
  final String _pageId;
  final String _targetType;
  final bool _isFeed;
  final int _viewId;
  final TaboolaClassicListener _taboolaStandardListener;
  final ScrollController scrollController;

  @override
  State<StatefulWidget> createState() => TaboolaClassicUnitState();
}

class TaboolaClassicUnitState extends State<TaboolaClassicUnit>
    with WidgetsBindingObserver {
  WebViewManager _webViewManager;
  TaboolaWebView _taboolaWebView;

  double _webViewHeight;
  bool _didCallScrollListener = false;
  GlobalKey _key = GlobalKey();

  @override
  void initState() {
    super.initState();

    _webViewHeight = 100;
    _webViewManager = WebViewManager(widget._extraPropertiesManager);
    _taboolaWebView = TaboolaWebView(
      widget._publisherId,
      widget._placement,
      widget._mode,
      widget._pageType,
      widget._pageUrl,
      widget._pageId,
      widget._targetType,
      _webViewManager,
      widget._viewId,
    );

    _webViewManager.openItemBrowser = openItemBrowser;
    _webViewManager.didResize = taboolaStandardDidResize;
    _webViewManager.didShow = taboolaStandardDidShow;
    _webViewManager.didFailToLoad =
        widget._taboolaStandardListener.didFailToLoad;
    _webViewManager.didClickOnItem =
        widget._taboolaStandardListener.didClickOnItem;

    widget.scrollController.addListener(callScrollListener);
    WidgetsBinding.instance.addObserver(
        this); // Necessary to react to application lifecycle messages

    // Taboola Feed implementation is not handled well by WebView texture rendering implementation. Despite limiting the WebView size, the texture rendered keeps growing, causing a crash.
    // Therefore Taboola requires using 'Hybrid Composition' rendering mode. According to WebView.platform documentaiton "Setting `platform` doesn't affect [WebView]s that were already created."
    // Notice: It is known that 'Hybrid Composition' comes at a cost of some performance on Android versions below 10.
    if (Platform.isAndroid) {
      WebView.platform = SurfaceAndroidWebView();
    }
  }

  @override
  void didUpdateWidget(TaboolaClassicUnit oldWidget) {
    super.didUpdateWidget(oldWidget);
    configureScrollController(oldWidget);
  }

  /// This method will mostly switch which ScrollController the TaboolaWebView will be working with (The internal one or the parent, external Widget one).
  void configureScrollController(TaboolaClassicUnit oldWidget) {
    if (oldWidget.scrollController != widget.scrollController) {
      oldWidget.scrollController.removeListener(
          callScrollListener); //TODO: check what happens if removeListener gets a null here
    }

    // Replace existing scroll listener
    widget.scrollController.removeListener(callScrollListener);
    widget.scrollController.addListener(callScrollListener);
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    super.dispose();
  }

  @override
  void didChangeMetrics() {
    // This is a fix for the bug that sometime the listener of the scroll doesnt work
    callScrollListenerIfListenerNotWorking();
  }

  void callScrollListener() {
    _didCallScrollListener = true;
    scrollListener();
  }

  void callScrollListenerIfListenerNotWorking() {
    if (!_didCallScrollListener) {
      scrollListener();
    }
  }

  @override
  Widget build(BuildContext context) {
    return ConstrainedBox(
      key: _key,
      constraints: BoxConstraints(maxHeight: _webViewHeight),
      child: _taboolaWebView,
    );
  }

  void taboolaStandardDidShow(String placement) {
    widget._taboolaStandardListener.didShow(placement);
  }

  void taboolaStandardDidResize(String placement, double height) {
    double maxHeight = DeviceDescriptor.getScreenSize()[1];
    if (maxHeight < height && Platform.isAndroid) {
      height = maxHeight;
    }
    if (_webViewHeight != height) {
      setState(() {
        _webViewHeight = height;
        widget._taboolaStandardListener.didResize(placement, _webViewHeight);
      });
    }
  }

  void scrollListener() {
    _taboolaWebView.scrollListener(
      widget.scrollController,
      widget._isFeed,
      _webViewHeight == DeviceDescriptor.getScreenSize()[1],
    );
  }

  void openItemBrowser(String itemUrl) {
    Logger.log("open item url: $itemUrl");
    Navigator.push(
      context,
      MaterialPageRoute(builder: (context) => ItemBrowser(itemUrl)),
    );
  }
}

==> taboola_sdk-0.3.0.tar.gz: lib/classic/taboola_classic.dart <==
import 'dart:io' show Platform;
import 'package:flutter/material.dart';
import 'package:taboola_sdk/descriptors/device_descriptor.dart';
import 'package:taboola_sdk/classic/webview_manager.dart';
import 'package:taboola_sdk/classic/taboola_webview.dart';
import 'package:taboola_sdk/classic/taboola_classic_listener.dart';
import '../extra_properties_manager.dart';
import 'itemBrowser.dart';
import 'package:taboola_sdk/utils/logger.dart';

class TaboolaClassicUnit extends StatefulWidget {
  TaboolaClassicUnit._(
    this._extraPropertiesManager,
    this._publisherId,
    this._mode,
    this._placement,
    this._pageUrl,
    this._pageType,
    this._pageId,
    this._targetType,
    this._isFeed,
    this._taboolaStandardListener, {
    Key? key,
    viewId,
    ScrollController? scrollController,
  })  : this.scrollController = scrollController ??= ScrollController(),
        this._viewId = viewId ?? new DateTime.now().millisecondsSinceEpoch,
        super(
            key:
                key); //Note: '{} means scrollController is an optional argument, it allows publishers to set an external ScrollController from their scrolling parent. Otherwise, on default, we create an internal listener and ask the user to set it in their scrolling view.'

  factory TaboolaClassicUnit(
    ExtraPropertiesManager extraPropertiesManager,
    String publisherId,
    String mode,
    String placement,
    String pageUrl,
    String pageType,
    String pageId,
    String targetType,
    bool isFeed,
    TaboolaClassicListener taboolaClassicListener, {
    int? viewId,
    ScrollController? scrollController,
  }) {
    return TaboolaClassicUnit._(
      extraPropertiesManager,
      publisherId,
      mode,
      placement,
      pageUrl,
      pageType,
      pageId,
      targetType,
      isFeed,
      taboolaClassicListener,
      viewId: viewId,
      scrollController: scrollController,
    );
  }

  final ExtraPropertiesManager _extraPropertiesManager;
  final String _publisherId;
  final String _mode;
  final String _placement;
  final String _pageUrl;
  final String _pageType;
  final String _pageId;
  final String _targetType;
  final bool _isFeed;
  final int _viewId;
  final TaboolaClassicListener _taboolaStandardListener;
  final ScrollController scrollController;

  @override
  State<StatefulWidget> createState() => TaboolaClassicUnitState();
}

class TaboolaClassicUnitState extends State<TaboolaClassicUnit>
    with WidgetsBindingObserver {
  late WebViewManager _webViewManager;
  late TaboolaWebView _taboolaWebView;

  double _webViewHeight = 100;
  bool _didCallScrollListener = false;
  GlobalKey _key = GlobalKey();

  @override
  void initState() {
    super.initState();

    _webViewManager = WebViewManager(widget._extraPropertiesManager);
    _taboolaWebView = TaboolaWebView(
      widget._publisherId,
      widget._placement,
      widget._mode,
      widget._pageType,
      widget._pageUrl,
      widget._pageId,
      widget._targetType,
      _webViewManager,
      widget._viewId,
    );

    _webViewManager.openItemBrowser = openItemBrowser;
    _webViewManager.didResize = taboolaStandardDidResize;
    _webViewManager.didShow = taboolaStandardDidShow;
    _webViewManager.didFailToLoad =
        widget._taboolaStandardListener.didFailToLoad;
    _webViewManager.didClickOnItem =
        widget._taboolaStandardListener.didClickOnItem;

    widget.scrollController.addListener(callScrollListener);
    WidgetsBinding.instance.addObserver(this);
  }

  @override
  void didUpdateWidget(TaboolaClassicUnit oldWidget) {
    super.didUpdateWidget(oldWidget);
    configureScrollController(oldWidget);
  }

  /// This method will mostly switch which ScrollController the TaboolaWebView will be working with (The internal one or the parent, external Widget one).
  void configureScrollController(TaboolaClassicUnit oldWidget) {
    if (oldWidget.scrollController != widget.scrollController) {
      oldWidget.scrollController.removeListener(
          callScrollListener); //TODO: check what happens if removeListener gets a null here
    }

    // Replace existing scroll listener
    widget.scrollController.removeListener(callScrollListener);
    widget.scrollController.addListener(callScrollListener);
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    widget.scrollController.removeListener(callScrollListener);
    super.dispose();
  }

  @override
  void didChangeMetrics() {
    // This is a fix for the bug that sometime the listener of the scroll doesnt work
    callScrollListenerIfListenerNotWorking();
  }

  void callScrollListener() {
    _didCallScrollListener = true;
    scrollListener();
  }

  void callScrollListenerIfListenerNotWorking() {
    if (!_didCallScrollListener) {
      scrollListener();
    }
  }

  @override
  Widget build(BuildContext context) {
    return ConstrainedBox(
      key: _key,
      constraints: BoxConstraints(maxHeight: _webViewHeight),
      child: _taboolaWebView,
    );
  }

  void taboolaStandardDidShow(String placement) {
    widget._taboolaStandardListener.didShow(placement);
  }

  void taboolaStandardDidResize(String placement, double height) {
    final screenSize = DeviceDescriptor.getScreenSize();
    if (screenSize == null || screenSize.isEmpty) {
      return;
    }
    double maxHeight = screenSize[1];
    if (maxHeight < height) {
      height = maxHeight;
    }

    if (_webViewHeight != height) {
      setState(() {
        _webViewHeight = height;
        widget._taboolaStandardListener.didResize(placement, _webViewHeight);
      });
    }
  }

  void scrollListener() {

    bool isScrollEnabled = DeviceDescriptor.getScreenSize() != null
        ? _webViewHeight == DeviceDescriptor.getScreenSize()![1]
        : false;

    _taboolaWebView.scrollListener(
        widget.scrollController, widget._isFeed, isScrollEnabled, context);
  }

  void openItemBrowser(String itemUrl) {
    Logger.log("open item url: $itemUrl");
    Navigator.push(
      context,
      MaterialPageRoute(builder: (context) => ItemBrowser(itemUrl)),
    );
  }
}
